/***************************************************************************
 *            fd.c
 *
 *  Sun Dec  5 22:25:24 2004
 *  Copyright  2004  Marcin K. Krzywonos
 *  mkk@gnutu.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, w
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 

#include "var.h"
#include "fdw.h"
#include "gtudb.h"


/*
 * Funkcja dodaje do listy wolnych dni nowa pozycje
 * parametry: date - data; desc - opis wolnego dnia
 */
int fdw_free_days_clist_add (type_date date, gchar *desc)
{
	gchar tmp [10];
	sprintf (tmp, "%d-%d-%d", date.day, date.month, date.year);
	
	GtkWidget *fdw_free_days_clist = (gpointer) lookup_widget (GTK_WIDGET (fdw_widget), "fdw_free_days_clist");
	gchar     *ins_str []          = {tmp, desc};
	
	gtk_clist_append (GTK_CLIST (fdw_free_days_clist), ins_str);
	gtk_clist_sort   (GTK_CLIST (fdw_free_days_clist));
	
	return 0;
}


/*
 * Funkcja usuwa z listy wpis o numerze 'id'
 */
int fdw_remove_free_day (int id)
{
	if (id == -1)
		return -1;
	
	GtkWidget *fdw_free_days_clist = (gpointer) lookup_widget (GTK_WIDGET (fdw_widget), "fdw_free_days_clist");
	gchar     *tmp;
	type_date  tmp_date;
	
	gtk_clist_get_text (GTK_CLIST (fdw_free_days_clist), id, 0, &tmp);
	sscanf (tmp, "%d-%d-%d", &tmp_date.day, &tmp_date.month, &tmp_date.year);
	
	gtudb_fd_list = g_slist_remove_link (gtudb_fd_list, g_slist_nth (gtudb_fd_list, gtudb_fd_search_day (tmp_date)));
	gtk_clist_remove (GTK_CLIST (fdw_free_days_clist), id);
	
	return 0;
}


/*
 * Funkcja wczytuje i wstawia do listy fdw_free_days_clist liste wolnych dni
 */
int fdw_free_days_clist_load ()
{
	GtkWidget *fdw_free_days_clist = (gpointer) lookup_widget (GTK_WIDGET (fdw_widget), "fdw_free_days_clist");
	gtk_clist_freeze (GTK_CLIST (fdw_free_days_clist));
	gtk_clist_clear  (GTK_CLIST (fdw_free_days_clist));
	
	GTUDBFreeDays *tmp_entry;
	int            i = 0;
	
	while (i < g_slist_length (gtudb_fd_list)) {
		tmp_entry = g_slist_nth (gtudb_fd_list, i)->data;
		fdw_free_days_clist_add (tmp_entry->date, g_strdup (tmp_entry->desc));
		
		i++;
	}
	
	gtk_clist_thaw (GTK_CLIST (fdw_free_days_clist));
}


/*
 * Funkcja wczytuje wolny dzien do okna 'edit_free_days_window'
 */
int efdw_edit_day ()
{
	GtkWidget     *efdw_desc_entry     = (gpointer) lookup_widget (GTK_WIDGET (efdw_widget), "efdw_desc_entry");
	GtkWidget     *efdw_date_gde       = (gpointer) lookup_widget (GTK_WIDGET (efdw_widget), "efdw_date_gde");
	GtkWidget     *fdw_free_days_clist = (gpointer) lookup_widget (GTK_WIDGET (fdw_widget), "fdw_free_days_clist");
	GTUDBFreeDays *tmp_entry;
	type_date      tmp_date;
	gchar         *tmp;
	
	gtk_clist_get_text (GTK_CLIST (fdw_free_days_clist), fdw_free_days_clist_rowselected, 0, &tmp);
	sscanf (tmp, "%d-%d-%d", &tmp_date.day, &tmp_date.month, &tmp_date.year);
	
	tmp_entry = g_slist_nth (gtudb_fd_list, gtudb_fd_search_day (tmp_date))->data;
	
	gtk_entry_set_text (GTK_ENTRY (efdw_desc_entry), tmp_entry->desc);
	gnome_date_edit_set_time (GNOME_DATE_EDIT (efdw_date_gde), typedate_to_timet (tmp_entry->date));
	
	return 0;
}


/*
 * Funkcja pobiera informacje o wolnym dniu z okna jego edycji a nastepnie
 * wstawia go do listy wolnych dni.
 */
int efdw_add_free_day ()
{
	GtkWidget     *efdw_desc_entry = (gpointer) lookup_widget (GTK_WIDGET (efdw_widget), "efdw_desc_entry");
	GtkWidget     *efdw_date_gde   = (gpointer) lookup_widget (GTK_WIDGET (efdw_widget), "efdw_date_gde");
	GTUDBFreeDays *tmp_entry       = g_new (GTUDBFreeDays, 1);
	type_date      tmp_date        = timet_to_typedate (gnome_date_edit_get_time (GNOME_DATE_EDIT (efdw_date_gde)));
	
	tmp_entry->date.day   = tmp_date.day;
	tmp_entry->date.month = tmp_date.month;
	tmp_entry->date.year  = tmp_date.year;
	
	tmp_entry->desc = strdup ((gchar *) gtk_entry_get_text (GTK_ENTRY (efdw_desc_entry)));
	
	if (gtudb_fd_search_day (tmp_date) == -1) {
		gtudb_fd_list = g_slist_append (gtudb_fd_list, tmp_entry);
		fdw_free_days_clist_add (tmp_entry->date, tmp_entry->desc);
	}
}
