/***************************************************************************
 *            db.c
 *
 *  Fri Sep  3 19:20:13 2004
 *  Copyright  2004  Marcin K. Krzywonos
 *  mkk@gnutu.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gnome.h>
#include <stdio.h>

#include "var.h"
#include "gtudb.h"
#include "idbw.h"


/*
 * Funkcja wczytuje plik bazy danych do listy gtudb_list
 */
int gtudb_read ()
{
	// Otwieramy plik
	FILE *db_file = fopen (gtu_file_name ("data.v2.gtu"), "r");
	
	GTUDBEntry   *tmp_entry;
	GTUDBSubject *tmp_subjects;
	gtudb_list = NULL;
	
	if ((!g_file_exists(gtu_file_name ("data.v2.gtu"))) && (g_file_exists (gtu_file_name ("dane.gtu"))))
		return 1;
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie i tworzymy
	// katalog programu (moze nie istniec !)
	if (db_file == NULL) {
		printf (_("BLAD: Nie mozna otworzyc pliku bazy danych (data.v2.gtu) !\n"));
		system ("mkdir ~/.gnutu");
	} else {
		gchar    line[8192], rtn[8192], *field3;
		int      field1, field2;
		
		while (!feof (db_file)) {
			if (line[0] == '.') {
				sprintf (rtn, "");
				
				sscanf (line, ".%d.%d %[^\n]\n", &field1, &field2, rtn); // Skanuje pojedyncza linie
				
				if (field1 == 1) { // Pole 1 - pobieramy date
					tmp_entry  = g_new (GTUDBEntry, 1);
					tmp_entry->subjects = NULL;
					
					sscanf  (rtn, "%d-%d-%d", &tmp_entry->date.day, &tmp_entry->date.month, &tmp_entry->date.year);
				}
				
				if (field1 == 2) { // Pole 2 - pobieramy przedmioty
					tmp_subjects = g_new (GTUDBSubject, 1);
					tmp_subjects->name = g_strdup (rtn);
				}
				
				if (field1 == 3) { // Pole 3 - pobieramy oceny
					tmp_subjects->marks = g_strdup (rtn);
				}
				
				if (field1 == 4) { // Pole 4 - pobieramy flagi sprawdzianow
					tmp_subjects->flag = atoi (rtn);
				}
				
				if (field1 == 5) { // Pole 5 - pobieramy godziny
					tmp_subjects->time = g_strdup (rtn);
					tmp_entry->subjects = g_slist_append (tmp_entry->subjects, tmp_subjects);
				}
				
				if (field1 == 6) { // Pole 6 - pobieramy notatki
					tmp_entry->notes = g_strdup (gtudb_text_to_notes (g_strdup (rtn)));
					gtudb_list = g_slist_append (gtudb_list, tmp_entry);
				}
			}
			
			
			fgets (line, 4096, db_file);
		}
	
		fclose (db_file); // zamykamy plik
	}
}


/*
 * Funkcja zapisuje baze danych do pliku
 */
int gtudb_write ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/data.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "w");
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie i tworzymy
	// katalog programu (moze nie istniec !)
	if (db_file == NULL) {
		system ("mkdir ~/.gnutu");
		db_file = fopen (tmp, "w");
	}
	
	if (db_file == NULL) {
		g_printf (_("BLAD: Nie mozna zapisac pliku bazy danych (dane.v2.gtu) !\n"));
		
		return 1;
	} else {
		int cnt     = 0;
		int cnt_sub = 0;
		
		GTUDBEntry   *tmp_entry;
		GTUDBSubject *tmp_subjects;
		GSList       *tmp_list_entry, *tmp_list_subjects;
		
		while (cnt < g_slist_length (gtudb_list)) {
			tmp_list_entry = g_slist_nth (gtudb_list, cnt);
			tmp_entry      = tmp_list_entry->data;
			
			fprintf (db_file, ".1.1 %d-%d-%d\n", tmp_entry->date.day, tmp_entry->date.month, tmp_entry->date.year);
			
			cnt_sub = 0;
			while (cnt_sub < g_slist_length (tmp_entry->subjects)) {
				tmp_list_subjects = g_slist_nth (tmp_entry->subjects, cnt_sub);
				tmp_subjects      = tmp_list_subjects->data;
				
				fprintf (db_file, ".2.%d %s\n", (cnt_sub + 1), tmp_subjects->name);
				fprintf (db_file, ".3.%d %s\n", (cnt_sub + 1), tmp_subjects->marks);
				fprintf (db_file, ".4.%d %d\n", (cnt_sub + 1), tmp_subjects->flag);
				fprintf (db_file, ".5.%d %s\n", (cnt_sub + 1), tmp_subjects->time);
				
				cnt_sub++;
			}
			
			fprintf (db_file, ".6.1 %s\n\n", gtudb_notes_to_text (tmp_entry->notes));
			
			cnt++;
		}
		
		fclose (db_file);
	}
	
	return 0;
}


/* Funkcja wyszukuje wpis
 * Paramerty:
 *            - date - wyszukiwana data
 * Zwraca numer wyszukanego wpisu, lub 0, jesli wpis nie istnieje
 */
int gtudb_search_entry (type_date date)
{
	int cnt = 0;
	int end = g_slist_length (gtudb_list);
	
	GSList     *tmp_db;
	GTUDBEntry *tmp_entry;
	
	while (cnt < end) {
		tmp_db = g_slist_nth (gtudb_list, cnt);
		tmp_entry = tmp_db->data;
		
		if (tmp_entry->date.day == 0)
			return -1;
		
		if ((tmp_entry->date.day   == date.day) && (tmp_entry->date.month == date.month) && (tmp_entry->date.year  == date.year))
			return cnt;
		
		cnt++;
	}
	
	return -1;
}


/*
 * Funkcja wyszukuje najblizszych sprawdzianow od date_begin przez "days" dni.
 */
GSList *gtudb_search_test (type_date date_begin, type_date date_end)
{	
	int cnt = 0;
	int end = g_slist_length (gtudb_list);
	
	GSList     *tmp_db = NULL;
	GSList     *rtn    = NULL;
	GTUDBEntry *tmp_entry;
	gchar       txt_date [20];
	
	while (cnt < end) {
		tmp_db = g_slist_nth (gtudb_list, cnt);
		tmp_entry = tmp_db->data;
		
		if ((g_date_compare (type_date_to_gdate (date_begin), type_date_to_gdate (tmp_entry->date)) <= 0) &&
		(g_date_compare (type_date_to_gdate (date_end), type_date_to_gdate (tmp_entry->date)) >= 0)) {
			if (gtudb_entry_search_tests (tmp_entry->date) > 0) {
				sprintf (txt_date, "%d-%d-%d", tmp_entry->date.day, tmp_entry->date.month, tmp_entry->date.year);
				rtn = g_slist_prepend (rtn, g_strdup (txt_date));
			}
		}
		
		cnt++;
	}
	
	rtn = g_slist_sort (rtn, date_compare);
}


/*
 * Funkcja sprawdza czy w dniu "date" jest jakis sprawdzian lub kartkowka.
 * Jesli jest sprawdzian funkcja zwraca wartosc 2, jesli kartkowka wartosc 1,
 * jesli nie ma nic zostaje zwrocona wartosc 0
 */
int gtudb_entry_search_tests (type_date date)
{
	GSList       *tmp_list  = g_slist_nth (gtudb_list, gtudb_search_entry (date));
	GTUDBEntry   *str_entry = tmp_list->data;
	GTUDBSubject *str_sub;
	GSList       *tmp_sub   = NULL;
	
	int cnt = 0;
	int end = g_slist_length (str_entry->subjects);
	
	while (cnt < end) {
		tmp_sub   = g_slist_nth (str_entry->subjects, cnt);
		str_sub   = tmp_sub->data;
		
		if (str_sub->flag > 0) {
			if (str_sub->flag == 1)
				return 1;
			else
				return 2;
		}
		
		cnt++;
	}
	
	return 0;
}


/*
 * Funkcja tworzy liste sprawdzianow dla dnia "date".
 * Parametr "flag" mowi o tym, czy szukac kartkowek, sprawdzianow
 */
GSList *gtudb_entry_get_subjects_list (type_date date, int flag)
{
	GSList       *tmp_list  = g_slist_nth (gtudb_list, gtudb_search_entry (date));
	GTUDBEntry   *str_entry = tmp_list->data;
	GTUDBSubject *str_sub;
	GSList       *tmp_sub   = NULL;
	GSList       *rtn       = NULL;
	
	if ((flag < 0) || (flag > 2))
		return rtn;
	
	int cnt = 0;
	int end = g_slist_length (str_entry->subjects);
	
	while (cnt < end) {
		tmp_sub   = g_slist_nth (str_entry->subjects, cnt);
		str_sub   = tmp_sub->data;
		
		if (str_sub->flag == flag)
			rtn = g_slist_append (rtn, g_strdup (str_sub->name));
		
		cnt++;
	}
	
	return rtn;
}


/*
 * Funkcja wczytuje plan lekcji do listy "gtudb_plan_list"
 */
int gtudb_plan_read ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/plan.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "r");
	
	if (db_file == NULL) {
		printf (_("BLAD: Nie mozna otworzyc pliku planu lekcji (plan.v2.gtu) !\n"));
	} else {
		gchar    line[8192], rtn[8192], *field3;
		int      field1, field2;
		
		while (!feof (db_file)) {
			if (line[0] == '.') {
				sprintf (rtn, "");
				sscanf (line, ".%d.%d %[^\n]\n", &field1, &field2, rtn); // Skanuje pojedyncza linie
				
				if (field2 == 1)
					gtudb_plan_list [field1] = NULL;
				
				gtudb_plan_list [field1] = g_slist_append (gtudb_plan_list [field1], g_strdup (rtn));
			}
			
			
			fgets (line, 4096, db_file);
		}
		
		fclose (db_file); // zamykamy plik
	}
}


/*
 * Funkcja zapisuje plan lekcji
 */
int gtudb_plan_write ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/plan.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "w");
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie
	if (db_file == NULL) {
		g_printf (_("BLAD: Nie mozna zapisac pliku planu lekcji (dane.v2.gtu) !\n"));
		
		return 1;
	} else {
		int i = 0;
		int j = 0;
		
		while (i < 7) {
			j = 0;
			while (j < g_slist_length (gtudb_plan_list [i])) {
				fprintf (db_file, ".%d.%d %s\n", i, (j + 1), g_strdup ((gchar *) g_slist_nth (gtudb_plan_list [i], j)->data));
				j++;
			}
			
			fprintf (db_file, "\n");
			
			i++;
		}
		
		fclose (db_file);
	}
	
	return 0;
}


/*
 * Funkcja wpisuje w planie lekcji numery godziny
 */
int gtudb_plan_num_hours (gboolean save)
{
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "01");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "02");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "03");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "04");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "05");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "06");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "07");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "08");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "09");
	gtudb_plan_list[0] = g_slist_append (gtudb_plan_list[0], "10");
	
	if (save == TRUE)
		gtudb_plan_write ();
	
	return 0;
}


/*
 * Funkcja odczytuje baze ustawien
 */
int gtudb_settings_read ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/settings.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "r");
	
	GTUDBSettings *tmp_entry;
	gtudb_settings_list = NULL;
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie
	if (db_file == NULL) {
		g_printf (_("BLAD: Nie mozna otworzyc pliku ustawien (settings.v2.gtu) !\n"));
		return 1;
	} else {
		gchar    line[8192], key[100], value[2048];
		
		while (!feof (db_file)) {
			if (line[0] == '.') {
				tmp_entry = g_new (GTUDBSettings, 1);
				sscanf (line, ".%s %[^\n]\n", key, value); // Skanuje pojedyncza linie
				tmp_entry->key   = g_strdup (key);
				tmp_entry->value = g_strdup (value);
				
				gtudb_settings_list = g_slist_append (gtudb_settings_list, tmp_entry);
			}
			
			fgets (line, 4096, db_file);
		}
		
		fclose (db_file); // zamykamy plik
	}
	
	return 0;
}


/*
 * Funkcja zapisuje baze ustawien
 */
int gtudb_settings_write ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/settings.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "w");
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie
	if (db_file == NULL) {
		g_printf (_("BLAD: Nie mozna zapisac pliku ustawien (settings.v2.gtu) !\n"));
		
		return 1;
	} else {
		int i = 0;
		GTUDBSettings *tmp_entry;
		
		while (i < g_slist_length (gtudb_settings_list)) {
			tmp_entry = g_slist_nth (gtudb_settings_list, i)->data;
			fprintf (db_file, ".%s %s\n", tmp_entry->key, tmp_entry->value);
			
			i++;
		}
		
		fclose (db_file);
	}
	
	return 0;
}


/*
 * Funkcja zwraca z bazy ustawien wartosc klucza "key"
 */
gchar *gtudb_settings_get_value (gchar *key)
{
	int i = 0;
	GTUDBSettings *tmp_entry;
	
	while (i < g_slist_length (gtudb_settings_list)) {
		tmp_entry = g_new (GTUDBSettings, 1);
		tmp_entry = g_slist_nth (gtudb_settings_list, i)->data;
		if (strcmp (tmp_entry->key, key) == 0)
			return g_strdup (tmp_entry->value);
		
		i++;
	}
	
	return "";
}


/*
 * Funkcja ustawia w bazie ustawien wartosc klucza "key" na "value"
 * Jesli klucz nie istnieje, to zostaje on utworzony
 */
int gtudb_settings_set_value (gchar *key, gchar *value)
{
	int i = 0;
	GTUDBSettings *tmp_entry;
	
	while (i < g_slist_length (gtudb_settings_list)) {
		tmp_entry = g_new (GTUDBSettings, 1);
		tmp_entry = g_slist_nth (gtudb_settings_list, i)->data;
		
		if (strcmp (tmp_entry->key, key) == 0) {
			gtudb_settings_list = g_slist_remove_link (gtudb_settings_list, g_slist_nth (gtudb_settings_list, i));
			
			tmp_entry->value = g_strdup (value);
			gtudb_settings_list = g_slist_append (gtudb_settings_list, tmp_entry);
			
			return 0;
		}
		
		i++;
	}
	
	tmp_entry = g_new (GTUDBSettings, 1);
	tmp_entry->key   = g_strdup (key);
	tmp_entry->value = g_strdup (value);
	gtudb_settings_list = g_slist_append (gtudb_settings_list, tmp_entry);
	
	return 0;
}


/*
 * Funkcja odczytuje zawartosc listy gtudb_fd_list z pliku
 */
int gtudb_fd_read ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/free_days.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "r");
	
	GTUDBFreeDays *tmp_entry;
	gtudb_fd_list = NULL;
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie
	if (db_file == NULL) {
		g_printf (_("BLAD: Nie mozna otworzyc pliku wolnych dni (free_days.v2.gtu) !\n"));
	} else {
		gchar     line[8192];
		type_date date;
		gchar     desc[4096];
		
		while (!feof (db_file)) {
			if (line[0] == '.') {
				tmp_entry = g_new (GTUDBFreeDays, 1);
				sscanf (line, ".%d-%d-%d %[^\n]\n", &date.day, &date.month, &date.year, desc); // Skanuje pojedyncza linie
				
				if (desc[0] == '|')
					strcpy ((gchar *) desc, "");
				
				tmp_entry->date = date;
				tmp_entry->desc = g_strdup (desc);
				
				gtudb_fd_list = g_slist_append (gtudb_fd_list, tmp_entry);
			}
			
			fgets (line, 8192, db_file);
		}
		
		fclose (db_file); // zamykamy plik
	}
	
	return 0;
}


/*
 * Funkcja zapisuje zawartosc listy gtudb_fd_list do pliku
 */
int gtudb_fd_write ()
{
	char  tmp [300];
	sprintf (tmp, "%s/.gnutu/free_days.v2.gtu", user_dir);
	
	// Otwieramy plik
	FILE *db_file = fopen (tmp, "w");
	
	// Jezeli nie udalo sie otworzyc pliku to komunikat o bledzie
	if (db_file == NULL) {
		g_printf (_("BLAD: Nie mozna zapisac pliku wolnych dni (free_days.v2.gtu) !\n"));
		
		return 1;
	} else {
		int i = 0;
		GTUDBFreeDays *tmp_entry;
		
		while (i < g_slist_length (gtudb_fd_list)) {
			tmp_entry = g_slist_nth (gtudb_fd_list, i)->data;
			fprintf (db_file, ".%d-%d-%d %s\n", tmp_entry->date.day, tmp_entry->date.month, tmp_entry->date.year, tmp_entry->desc);
			
			i++;
		}
		
		fclose (db_file);
	}
	
	return 0;
}


/*
 * Funkcja wyszukuje dzien 'date' w bazie wolnych dni.
 */
int gtudb_fd_search_day (type_date date)
{
	int            i = 0;
	GTUDBFreeDays *tmp_entry;
	
	while (i < g_slist_length (gtudb_fd_list)) {
		tmp_entry = g_slist_nth (gtudb_fd_list, i)->data;
		if ((tmp_entry->date.day == date.day) && (tmp_entry->date.month == date.month) && (tmp_entry->date.year == date.year))
			return i;
		
		i++;
	}
	
	return -1;
}


/*
 * Funkcja usuwa w tekscie znaki "|" i zamienia je na "\n"
 */
gchar *gtudb_text_to_notes (gchar *text)
{
	int    cnt = 0;
	
	while (cnt < strlen (text)) {
		if (text[cnt] == '|')
			text[cnt] = '\n';
		
		cnt++;
	}
	
	return text;
}


/*
 * Funkcja zamienia w tekscie znaki "\n" i zamienia je na "|"
 */
gchar *gtudb_notes_to_text (gchar *text)
{
	int    cnt = 0;
	
	while (cnt < strlen (text)) {
		if (text[cnt] == '\n') {
			text[cnt] = '|';
		} else {
			if (text[cnt] == '|')
				text[cnt] = ' ';
		}
		
		cnt++;
	}
	
	return text;
}
