/***************************************************************************
 *            idbw.c
 *
 *  Sun Dec 19 21:52:12 2004
 *  Copyright  2004  Marcin K. Krzywonos
 *  mkk@gnutu.org
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <gnome.h>
#include "idbw.h"
#include "var.h"


/*
 * Funkcja wyswietla okno 'import_db_window'
 */
int idbw_show ()
{
	idbw_widget = (gpointer) create_import_db_window ();
	gtk_widget_show (GTK_WIDGET (idbw_widget));
	
	// importujemy ustawienia...
	idbw_import ();
	
	GtkWidget *idbw_btn_cancel = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_btn_cancel");
	gtk_widget_set_sensitive (GTK_WIDGET (idbw_btn_cancel), TRUE);
	
	return 0;
}


/*
 * Funkcja etykiete o numerze 'id' jako aktywna (pogrubiona czcionka).
 */
int idbw_active_label (int id, gboolean active)
{
	if ((id < 1) || (id > 5))
		return 1;
	
	GtkWidget *widget;
	
	switch (id) {
		case 1: widget = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_info_label_import_db"); break;
		case 2: widget = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_info_label_plan"); break;
		case 3: widget = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_info_label_diary"); break;
		case 4: widget = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_info_label_free_days"); break;
		case 5: widget = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_info_label_loading_settings"); break;
	}
	
	gchar *tmp1;
	gchar  tmp2 [1024];
	
	if (active) {
		tmp1 = (gchar *) gtk_label_get_label (GTK_LABEL (widget));
		sprintf (tmp2, "<b>%s</b>", tmp1);
		
		gtk_label_set_label (GTK_LABEL (widget), tmp2);
	} else {
		tmp1 = (gchar *) gtk_label_get_label (GTK_LABEL (widget));
		
		if (tmp1 [0] == '<') {
			sscanf (tmp1, "<b>%[^<]</b>", tmp2);
			gtk_label_set_label (GTK_LABEL (widget), tmp2);
		}
	}
	
	while (gtk_events_pending ())
		gtk_main_iteration ();
	
	return 0;
}


/*
 * Funkcja ustawia pasek postepu na 'percent' procent
 */
int idbw_progress_progress_bar_set_percent (int percent)
{
	GtkWidget *idbw_progress_progress_bar = (gpointer) lookup_widget (GTK_WIDGET (idbw_widget), "idbw_progress_progress_bar");
	
	float tmp_per = (gfloat) percent / 100;
	
	gchar tmp [10];
	sprintf (tmp, "%d %%", percent);
	
	gtk_progress_bar_set_fraction (GTK_PROGRESS_BAR (idbw_progress_progress_bar), tmp_per);
	gtk_progress_bar_set_text     (GTK_PROGRESS_BAR (idbw_progress_progress_bar), tmp);
	
	while (gtk_events_pending ())
		gtk_main_iteration ();
	
	return 0;
}


/*
 * Funkcja importuje stare dane
 */
int idbw_import ()
{
	idbw_import_db ();
	idbw_import_plan ();
	idbw_import_diary ();
	idbw_import_free_days ();
	idbw_import_load_settings ();
	
	return 0;
}


/*
 * Funkcja importuje stara baze danych
 */
int idbw_import_db ()
{
	idbw_active_label (1, TRUE);
	idbw_active_label (2, FALSE);
	idbw_active_label (3, FALSE);
	idbw_active_label (4, FALSE);
	idbw_active_label (5, FALSE);
	
	if (g_file_exists (gtu_file_name ("dane.gtu"))) {
		typedef struct {
			type_date date;
			char      test       [20];
			char      quiz1      [20];
			char      quiz2      [20];
			char      notes      [255];
			char      subjects   [8][20];
			char      marks      [9][20];
		} old_entry_struct;
		
		// importujemy stary plik
		FILE *file    = fopen (gtu_file_name ("dane.gtu"), "r");
		FILE *db_file = fopen (gtu_file_name ("data.v2.gtu"), "w");
		if ((file == NULL) || (db_file == NULL)) {
			return -1;
		} else {
			int              i = 0;
			old_entry_struct tmp;
			
			while (fread (&tmp, sizeof (tmp), 1, file)) {
				fprintf (db_file, ".1.1 %d-%d-%d\n", tmp.date.day, (tmp.date.month + 1), tmp.date.year);
				
				int j = 1;
				while (j < 8) {
					if (strlen (tmp.subjects [j]) <= 1) {
						j++;
						continue;
					}
					
					fprintf (db_file, ".2.%d %s\n", j, (gchar *) tmp.subjects [j]);
					fprintf (db_file, ".3.%d %s\n", j, (gchar *) tmp.marks [j]);
					
					if (strcmp (tmp.test, tmp.subjects [j]) == 0) {
						fprintf (db_file, ".4.%d 2\n", j);
						strcpy ((gchar *) tmp.subjects, "");
					} else {
						if ((strcmp (tmp.quiz1, tmp.subjects [j]) == 0) || (strcmp (tmp.quiz2, tmp.subjects [j]) == 0))
							fprintf (db_file, ".4.%d 1\n", j);
						else
							fprintf (db_file, ".4.%d 0\n", j);
					}
					
					fprintf (db_file, ".5.%d 0%d\n", j, j);
					
					j++;
				}
				
				fprintf (db_file, ".6.1 %s\n", gtudb_text_to_notes ((gchar *) tmp.notes));
				
				fputs ("\n", db_file);
			}
			
			fclose (file);
			fclose (db_file);
		}
	}
	
	
	idbw_progress_progress_bar_set_percent (20);
	
	return 0;
}


/*
 * Funkcja importuje stary plan lekcji
 */
int idbw_import_plan ()
{
	idbw_active_label (1, FALSE);
	idbw_active_label (2, TRUE);
	idbw_active_label (3, FALSE);
	idbw_active_label (4, FALSE);
	idbw_active_label (5, FALSE);
	
	
	if (g_file_exists (gtu_file_name ("plan.gtu"))) {
		typedef struct {
			char day [9][20];
		} old_plan_struct;
		
		// importujemy stary plik
		FILE *file    = fopen (gtu_file_name ("plan.gtu"), "r");
		FILE *db_file = fopen (gtu_file_name ("plan.v2.gtu"), "w");
		if ((file == NULL) || (db_file == NULL)) {
			return -1;
		} else {
			int             i = 1;
			int             j = 1;
			old_plan_struct tmp;
			
			while (j < 8) {
				fprintf (db_file, ".0.%d 0%d\n", j, j);
				j++;
			}
			
			while (i <= 5) {
				if (feof (file))
					break;
				
				fread (&tmp, sizeof (tmp), 1, file);
				
				j = 1;
				while (j < 8) {
					fprintf (db_file, ".%d.%d %s\n", i, j, (gchar *) tmp.day [j]);
					
					j++;
				}
				
				fputs ("\n", db_file);
				i++;
			}
			
			fclose (file);
			fclose (db_file);
		}
	}
	
	
	idbw_progress_progress_bar_set_percent (40);
	
	return 0;
}


/*
 * Funkcja importuje stary dziennik
 */
int idbw_import_diary ()
{
	idbw_active_label (1, FALSE);
	idbw_active_label (2, FALSE);
	idbw_active_label (3, TRUE);
	idbw_active_label (4, FALSE);
	idbw_active_label (5, FALSE);
	
	system ("cp ~/.gnutu/dziennik ~/.gnutu/diary -R");
	
	idbw_progress_progress_bar_set_percent (60);
	
	return 0;
}


/*
 * Funkcja importuje stara baze wolnych dni
 */
int idbw_import_free_days ()
{
	idbw_active_label (1, FALSE);
	idbw_active_label (2, FALSE);
	idbw_active_label (3, FALSE);
	idbw_active_label (4, TRUE);
	idbw_active_label (5, FALSE);
	
	
	if (g_file_exists (gtu_file_name ("wolne_dni.gtu"))) {
		// importujemy stary plik
		FILE *file    = fopen (gtu_file_name ("wolne_dni.gtu"), "r");
		FILE *db_file = fopen (gtu_file_name ("free_days.v2.gtu"), "w");
		if ((file == NULL) || (db_file == NULL)) {
			return -1;
		} else {
			type_date tmp;
			
			while (fread (&tmp, sizeof (tmp), 1, file))
				fprintf (db_file, ".%d-%d-%d |\n", tmp.day, tmp.month, tmp.year);
			
			fputs ("\n", db_file);
			
			fclose (file);
			fclose (db_file);
		}
	}
	
	
	idbw_progress_progress_bar_set_percent (80);
	
	return 0;
}


/*
 * Funkcja wczytuje nowe ustawienia
 */
int idbw_import_load_settings ()
{
	idbw_active_label (1, FALSE);
	idbw_active_label (2, FALSE);
	idbw_active_label (3, FALSE);
	idbw_active_label (4, FALSE);
	idbw_active_label (5, TRUE);
	
	gtudb_read ();
	gtudb_plan_read ();
	gtudb_fd_read ();
	
	mw_edit_entry (date_today (), FALSE);
	
	idbw_progress_progress_bar_set_percent (100);
	
	return 0;
}
